@@ -1,19 +1,23 @@
+# -*- coding: utf-8 -*-
+
 import re
+from functools import update_wrapper
+from weakref import WeakSet
+
 from django.apps import apps
 from django.contrib.auth import REDIRECT_FIELD_NAME
 from django.core.exceptions import ImproperlyConfigured
+from django.db.models import Q
 from django.db.models.base import ModelBase
 from django.http import HttpResponseRedirect
 from django.template.response import TemplateResponse
-from django.urls import NoReverseMatch, reverse
+from django.urls import NoReverseMatch, reverse, reverse_lazy, resolve
 from django.utils.functional import LazyObject
 from django.utils.module_loading import import_string
 from django.utils.text import capfirst
 from django.utils.translation import gettext as _, gettext_lazy
 from django.views.decorators.cache import never_cache
 from django.views.decorators.csrf import csrf_protect
-from functools import update_wrapper
-from weakref import WeakSet
 
 from django_webix.admin_webix import ModelWebixAdmin
 
@@ -47,6 +51,8 @@ class AdminWebixSite:
     index_template = None
     login_template = None
     logout_template = None
+    dashboard_template = 'admin_webix/dashboard.js'
+    webgis_template = None
     password_change_template = None
     password_change_done_template = None
 
@@ -55,12 +61,18 @@ class AdminWebixSite:
         self.name = name
         all_sites.add(self)
 
+    def is_webgis_enable(self):
+        return apps.is_installed("django_webix_leaflet")
+
+    def is_webix_filter_enable(self):
+        return apps.is_installed("django_webix_filter")
+
     def has_permission(self, request):
         """
         Return True if the given HttpRequest has permission to view
         *at least one* page in the admin site.
         """
-        return request.user.is_active and request.user.is_staff
+        return request.user.is_active
 
     #    def check(self, app_configs): # TODO
     #        if app_configs is None:
@@ -117,6 +129,8 @@ class AdminWebixSite:
 
                 # Instantiate the admin class to save in the registry
                 self._registry[model] = admin_class(model, self)
+            # else:
+            #     raise Exception(model, 'errore swap')
 
     def unregister(self, model_or_iterable):
         """
@@ -216,6 +230,55 @@ class AdminWebixSite:
 
         return app_list
 
+    def available_menu_items(self, user):
+        from django_webix.admin_webix.models import WebixAdminMenu
+        queryset = WebixAdminMenu.objects.all()
+        if user.is_superuser:
+            out = queryset.values_list('id', flat=True)
+        else:
+            out = []
+            for el in queryset.filter(enabled=True).filter(Q(active_all=True) | Q(groups__in=user.groups.all())):
+                if el.model is not None:
+                    if user.has_perm(el.model.app_label + '.view_' + el.model.model):
+                        out.append(el.id)
+                else:
+                    out.append(el.id)
+        return out
+
+    def get_tree(self, items, available_items):
+        from django_webix.admin_webix.models import WebixAdminMenu
+        out = []
+        new_level = True
+        for item in items:
+            if item.id in available_items:
+                menu_item = {
+                    "id": "menu_{}".format(item),
+                    "value": "{}".format(item),
+                    "icon": item.icon if item.icon not in ['', None] else "fas fa-archive",
+                }
+                soons = WebixAdminMenu.objects.filter(parent=item, id__in=available_items).order_by('tree_id', 'lft')
+                children = self.get_tree(soons, available_items)
+                if children != []:
+                    menu_item["submenu"] = children
+                elif (item.model is not None) or (item.url not in ['', None]):
+                    if item.get_url is None:
+                        URL = ""
+                    else:
+                        URL = item.get_url
+                    menu_item["url"] = URL
+                    menu_item["loading_type"] = "js_script"
+                out.append(menu_item)
+        return out
+
+    def get_menu_list(self, request):
+        from django_webix.admin_webix.models import WebixAdminMenu
+        if request.user.is_anonymous:
+            return {}
+        available = self.available_menu_items(request.user)
+
+        return self.get_tree(WebixAdminMenu.objects.filter(level=0, id__in=available).order_by('tree_id', 'lft'),
+                             available)
+
     def admin_view(self, view, cacheable=False):
         """
         Decorator to create an admin view attached to this ``AdminWebixSite``. This
@@ -262,6 +325,8 @@ class AdminWebixSite:
         # Since this module gets imported in the application's root package,
         # it cannot import models from other applications at the module level,
         # and django.contrib.contenttypes.views imports ContentType.
+        from django_webix.admin_webix import forms
+        from django_webix.admin_webix import views
 
         def wrap(view, cacheable=False):
             def wrapper(*args, **kwargs):
@@ -272,24 +337,42 @@ class AdminWebixSite:
 
         # Admin-site-wide views.
         urlpatterns = [  # prefix = admin_webix:XXXX
+
             path('', wrap(self.index), name='index'),
             path('login/', self.login, name='login'),
             path('logout/', wrap(self.logout), name='logout'),
             path('dashboard/', wrap(self.dashboard), name='dashboard'),
-
+            path('index/', wrap(self.index), name='index'),
             path('password_change/', wrap(self.password_change, cacheable=True), name='password_change'),
-            path(
-                'password_change/done/',
-                wrap(self.password_change_done, cacheable=True),
-                name='password_change_done',
-            ),
-            # path('jsi18n/', wrap(self.i18n_javascript, cacheable=True), name='jsi18n'),
-            # path(
-            #    'r/<int:content_type_id>/<path:object_id>/',
-            #    wrap(contenttype_views.shortcut),
-            #    name='view_on_site',
-            # ),
+            path('password_change/done/', wrap(self.password_change_done, cacheable=True), name='password_change_done'),
+
+            # ################################################ user update ############################################
+
+            path('account/update/<int:pk>/', views.UserUpdate.as_view(), name='account_update'),
+
+            # ############################################ Reset password by email ####################################
+            # nessuna di queste view deve essere sotto wrap perche si deve poter accedere anche non essendo loggati
+            path('password_reset/', self.password_reset, name='password_reset'),
+            # OLD
+            # path('reset/<uidb64>/<token>/', self.password_reset_confirm, name='password_reset_confirm'),
+            # path('reset/<uidb64>/<token>/', PasswordResetConfirmView.as_view(
+            #     form_class=forms.WebixSetPasswordForm,
+            #     success_url=reverse_lazy('admin_webix:password_reset_complete'),
+            #     template_name='admin_webix/account/password_reset_confirm.html'
+            # ), name='password_reset_confirm'),
+            path('reset/<uidb64>/<token>/', views.PasswordResetConfirmViewCustom.as_view(
+                form_class=forms.WebixSetPasswordForm,
+                success_url=reverse_lazy('admin_webix:password_reset_complete'),
+                template_name='admin_webix/account/password_reset_confirm.html'
+            ), name='password_reset_confirm'),
+
+            path('password_reset/done/', self.password_reset_done, name='password_reset_done'),
+            path('reset/done/', self.password_reset_complete, name='password_reset_complete'),
         ]
+        if apps.is_installed("two_factor"):
+            urlpatterns += [
+                path('two_factor/', wrap(self.two_factor_profile), name='two_factor_profile'),
+            ]
 
         # Add in each model's views, and create a list of valid URLS for the
         # app_index
@@ -328,30 +411,41 @@ class AdminWebixSite:
             'site_title': self.site_title,
             'site_header': self.site_header,
             'site_url': site_url,
+            'is_webgis_enable': self.is_webgis_enable(),
+            'is_webix_filter_enable': self.is_webix_filter_enable(),
             'has_permission': self.has_permission(request),  # utils for menu
+            'menu_list': self.get_menu_list(request),  # utils for menu
             'available_apps': self.get_app_list(request),  # utils for menu
             'webix_container_id': self.webix_container_id,
         }
 
-    def dashboard(self, request, extra_context=None):  # TODO
-        return None
+    @never_cache
+    def dashboard(self, request, extra_context=None):
+        from django.views.generic import TemplateView
+        defaults = {
+            'extra_context': {**self.each_context(request), **(extra_context or {})},
+        }
+        if self.dashboard_template is not None:
+            defaults['template_name'] = self.dashboard_template
+        return TemplateView.as_view(**defaults)(request)
 
     def password_change(self, request, extra_context=None):
         """
         Handle the "change password" task -- both form display and validation.
         """
         from django.contrib.admin.forms import AdminPasswordChangeForm
-        from django.contrib.auth.views import PasswordChangeView
+        # from django.contrib.auth.views import PasswordChangeView
+        from django_webix.admin_webix.views import PasswordChangeViewCustom
         url = reverse('admin_webix:password_change_done', current_app=self.name)
         defaults = {
             'form_class': AdminPasswordChangeForm,
             'success_url': url,
             'extra_context': {**self.each_context(request), **(extra_context or {})},
+            'template_name': self.password_change_template or 'admin_webix/account/password_change.js',
         }
-        if self.password_change_template is not None:
-            defaults['template_name'] = self.password_change_template
+
         request.current_app = self.name
-        return PasswordChangeView.as_view(**defaults)(request)
+        return PasswordChangeViewCustom.as_view(**defaults)(request)
 
     def password_change_done(self, request, extra_context=None):
         """
@@ -366,6 +460,104 @@ class AdminWebixSite:
         request.current_app = self.name
         return PasswordChangeDoneView.as_view(**defaults)(request)
 
+    def password_reset(self, request, extra_context=None):
+        """
+        Handle the "reset password" task -- both form display and validation.
+        """
+
+        from django.contrib.auth.views import PasswordResetView
+        from django_webix.admin_webix import forms
+        from django.contrib.sites.shortcuts import get_current_site
+
+        current_site = get_current_site(request)
+        site_name = current_site.name
+        domain = current_site.domain
+
+        if extra_context is None:
+            extra_context = {}
+        extra_context['domain'] = domain
+        extra_context['site_name'] = site_name
+
+        template = 'admin_webix/account/password_reset_form.js'
+        if not self.has_permission(request):
+            template = 'admin_webix/account/password_reset_form.html'
+
+        defaults = {
+            'template_name': template,
+            'email_template_name': 'admin_webix/account/password_reset_email.html',
+            'form_class': forms.WebixPasswordResetForm,
+            'success_url': reverse_lazy('admin_webix:password_reset_done'),
+            'extra_context': {**self.each_context(request), **(extra_context or {})},
+        }
+
+        request.current_app = self.name
+        return PasswordResetView.as_view(**defaults)(request)
+
+    def password_reset_done(self, request, extra_context=None):
+        """
+        Handle the "reset password" task -- both form display and validation.
+        """
+
+        from django.contrib.auth.views import PasswordResetDoneView
+
+        template = 'admin_webix/account/password_reset_done.js'
+        if not self.has_permission(request):
+            template = 'admin_webix/account/password_reset_done.html'
+
+        defaults = {
+            'template_name': template,
+            'extra_context': {**self.each_context(request), **(extra_context or {})},
+        }
+
+        request.current_app = self.name
+        return PasswordResetDoneView.as_view(**defaults)(request)
+
+    # NON so perche non funziona se lo metto nella function
+    # def password_reset_confirm(self, request, extra_context=None):
+    #
+    #     from django.contrib.auth.views import PasswordResetConfirmView
+    #     from django_webix.admin_webix import forms
+    #
+    #     defaults = {
+    #         'template_name': 'admin_webix/account/password_reset_confirm.js',
+    #         'form_class': forms.WebixSetPasswordForm,
+    #         'success_url': reverse_lazy('admin_webix:password_reset_complete'),
+    #         # 'extra_context': {**self.each_context(request)},
+    #     }
+    #     # raise Exception(extra_context, defaults)
+    #     # request.current_app = self.name
+    #     return PasswordResetConfirmView.as_view(
+    #         form_class=forms.WebixSetPasswordForm,
+    #         success_url=reverse_lazy('admin_webix:password_reset_complete'),
+    #         template_name='admin_webix/account/password_reset_confirm.js'
+    #     )(request)
+
+    def password_reset_complete(self, request, extra_context=None):
+
+        from django.contrib.auth.views import PasswordResetCompleteView
+
+        defaults = {
+            'template_name': 'admin_webix/account/password_reset_complete.html',
+            'extra_context': {**self.each_context(request), **(extra_context or {})},
+        }
+
+        request.current_app = self.name
+        return PasswordResetCompleteView.as_view(**defaults)(request)
+
+    def two_factor_profile(self, request, extra_context=None):
+        if apps.is_installed("two_factor"):
+            from django_webix.views import WebixTemplateView
+
+            defaults = {
+                'template_name': 'admin_webix/account/two_factor.js',
+                'extra_context': {**self.each_context(request), **(extra_context or {})},
+            }
+
+            request.current_app = self.name
+            return WebixTemplateView.as_view(**defaults)(request)
+        else:
+            return None
+
     @never_cache
     def logout(self, request, extra_context=None):
         """
@@ -379,12 +571,13 @@ class AdminWebixSite:
                 # Since the user isn't logged out at this point, the value of
                 # has_permission must be overridden.
                 'has_permission': False,
+                'template_name': self.logout_template or 'admin_webix/logged_out.html',
+                'site_title': self.site_title,
                 **(extra_context or {})
             },
         }
-        if self.logout_template is not None:
-            defaults['template_name'] = self.logout_template
         request.current_app = self.name
+        LogoutView.template_name = self.logout_template or 'admin_webix/logged_out.html'
         return LogoutView.as_view(**defaults)(request)
 
     @never_cache
@@ -401,12 +594,37 @@ class AdminWebixSite:
         # Since this module gets imported in the application's root package,
         # it cannot import models from other applications at the module level,
         # and django.contrib.admin.forms eventually imports User.
-        from django.contrib.admin.forms import AdminAuthenticationForm
+        # from django.contrib.admin.forms import AdminAuthenticationForm
+        from django.contrib.auth.forms import AuthenticationForm
+
+        class AdminAuthenticationForm(AuthenticationForm):
+            """
+            A custom authentication form used in the admin app.
+            """
+            error_messages = {
+                **AuthenticationForm.error_messages,
+                'invalid_login': _(
+                    "Please enter the correct %(username)s and password for a staff "
+                    "account. Note that both fields may be case-sensitive."
+                ),
+            }
+            required_css_class = 'required'
+
+            def confirm_login_allowed(self, user):
+                super().confirm_login_allowed(user)
+                # if not user.is_staff:
+                #    raise forms.ValidationError(
+                #        self.error_messages['invalid_login'],
+                #        code='invalid_login',
+                #        params={'username': self.username_field.verbose_name}
+                #    )
+
         context = {
             **self.each_context(request),
             'title': _('Log in'),
             'app_path': request.get_full_path(),
             'username': request.user.get_username(),
+            'site_title': self.site_title,
         }
         if (REDIRECT_FIELD_NAME not in request.GET and
             REDIRECT_FIELD_NAME not in request.POST):
@@ -421,18 +639,35 @@ class AdminWebixSite:
         request.current_app = self.name
         return LoginView.as_view(**defaults)(request)
 
+    def extra_index_context(self, request):
+        return {}
+
     @never_cache
-    def index(self, request, extra_context=None):
+    def index(self, request, extra_context=None):  # TODO da terminare la parte del template in modo carino circa
         """
         Display the main admin index page, which lists all of the installed
         apps that have been registered in this site.
         """
-        app_list = self.get_app_list(request)
+        if self.is_webgis_enable() and self.webgis_template is None:
+            raise ImproperlyConfigured('Webgis template is not set')
+
+        history_url = request.GET.get('state', None)
+        try:
+            resolve(history_url)
+        except:
+            history_url = None
+
+        active_tab = request.GET.get('tab', None)
+        if active_tab not in ['webgis_leaflet', self.webix_container_id]:
+            active_tab = self.webix_container_id
 
         context = {
             **self.each_context(request),
+            **self.extra_index_context(request),
+            'history_url': history_url,
             'title': self.index_title,
-            'app_list': app_list,
+            'app_list': self.get_app_list(request),
+            'active_tab': active_tab,
             **(extra_context or {}),
         }
 
