@@ -1,7 +1,5 @@
 # -*- coding: utf-8 -*-
 
-from __future__ import unicode_literals, absolute_import
-
 import copy
 from collections import OrderedDict, defaultdict
 from json import dumps, loads
@@ -21,22 +19,27 @@ from django.core.exceptions import ImproperlyConfigured
 from django.core.serializers.json import DjangoJSONEncoder
 from django.db import models, connection
 from django.forms.forms import DeclarativeFieldsMetaclass
-from django.forms.models import ModelFormMetaclass, InlineForeignKeyField
+from django.forms.models import ModelFormMetaclass
 from django.forms.utils import ErrorList
 from django.urls import reverse
 from django.utils.encoding import force_text
 from django.utils.text import capfirst
+from django.utils.timezone import is_naive, make_naive
 from django.utils.translation import ugettext_lazy as _
 from random import randint
 from sorl.thumbnail import get_thumbnail
+from django.utils import formats
+
+if django.__version__ < '3.1':
+    try:
+        from django.contrib.postgres.forms.jsonb import JSONField
+    except ImportError:
+        JSONField = forms.Field
+elif django.__version__ >= '3.1':
+    from django.forms.fields import JSONField
 
 try:
-    from django.contrib.postgres.forms.jsonb import JSONField
-except ImportError:
-    JSONField = forms.Field
-
-try:
-    from django.contrib.postgres.fields.array import SimpleArrayField
+    from django.contrib.postgres.forms import SimpleArrayField
 except ImportError:
     SimpleArrayField = None
 
@@ -46,7 +49,7 @@ except ImportError:
     GeometryField = None
 
 
-class BaseWebixMixin(object):
+class BaseWebixMixin:
     form_fix_height = None
     min_count_suggest = 100
     style = 'stacked'
@@ -90,50 +93,59 @@ class BaseWebixMixin(object):
                 _field = None
             else:
                 del self.fields[readonly_field]
-                if not hasattr(self, 'instance') or self.instance.pk is None:
+
+                if isinstance(_field, django.db.models.fields.DateTimeField) or \
+                    isinstance(_field, django.forms.fields.DateTimeField) or \
+                    isinstance(_field, django.forms.fields.DateTimeInput):
+                    # TODO: it can be improved
                     self.fields[readonly_field] = forms.CharField(
-                        label=_(_field.verbose_name).capitalize(), required=False
+                        label=_(_field.verbose_name).capitalize(),
+                        required=False
                     )
-                else:
-                    if isinstance(_field, django.db.models.fields.DateTimeField) or \
-                        isinstance(_field, django.forms.fields.DateTimeField) or \
-                        isinstance(_field, django.forms.fields.DateTimeInput):
-                        # TODO: it can be improved
-                        self.fields[readonly_field] = forms.CharField(
-                            label=_(_field.verbose_name).capitalize(),
-                            required=False
-                        )
-                        if getattr(self.instance, readonly_field):
-                            value = '{}'.format(getattr(self.instance, readonly_field).strftime('%d/%m/%Y %H:%i'))
-                        else:
-                            value = ''
-                    elif isinstance(_field, django.db.models.fields.DateField) or \
-                        isinstance(_field, django.forms.fields.DateField):
-                        # TODO: it can be improved
-                        self.fields[readonly_field] = forms.CharField(
-                            label=_(_field.verbose_name).capitalize(),
-                            required=False
-                        )
-                        if getattr(self.instance, readonly_field):
-                            value = '{}'.format(getattr(self.instance, readonly_field).strftime('%d/%m/%Y'))
-                        else:
-                            value = ''
-                    elif isinstance(_field, django.db.models.fields.BooleanField) or \
-                        isinstance(_field, django.forms.fields.CheckboxInput):
-                        # TODO: it can be improved
-                        self.fields[readonly_field] = forms.BooleanField(
-                            label=_(_field.verbose_name).capitalize(),
-                            required=False
-                        )
+                    if hasattr(self, 'instance') and \
+                        self.instance.pk is not None and getattr(self.instance, readonly_field):
+                        _value = getattr(self.instance, readonly_field)
+                        if not is_naive(_value):
+                            _value = make_naive(_value)
+                        value = '{}'.format(_value.strftime('%d/%m/%Y %H:%M'))
+                    else:
+                        value = ''
+                elif isinstance(_field, django.db.models.fields.DateField) or \
+                    isinstance(_field, django.forms.fields.DateField):
+                    # TODO: it can be improved
+                    self.fields[readonly_field] = forms.CharField(
+                        label=_(_field.verbose_name).capitalize(),
+                        required=False
+                    )
+                    if hasattr(self, 'instance') and \
+                        self.instance.pk is not None and getattr(self.instance, readonly_field):
+                        value = '{}'.format(getattr(self.instance, readonly_field).strftime('%d/%m/%Y'))
+                    else:
+                        value = ''
+                elif isinstance(_field, django.db.models.fields.BooleanField) or \
+                    isinstance(_field, django.forms.fields.CheckboxInput):
+                    # TODO: it can be improved
+                    self.fields[readonly_field] = forms.BooleanField(
+                        label=_(_field.verbose_name).capitalize(),
+                        required=False
+                    )
+                    if hasattr(self, 'instance') and self.instance.pk is not None:
                         value = getattr(self.instance, readonly_field) or None
                     else:
-                        self.fields[readonly_field] = forms.CharField(
-                            label=_(_field.verbose_name).capitalize(),
-                            required=False
-                        )
-                        value = '{}'.format(getattr(self.instance, readonly_field) or '')
-                    self.fields[readonly_field].initial = value
-                    self.initial[readonly_field] = value
+                        value = None
+                else:
+                    self.fields[readonly_field] = forms.CharField(
+                        label=_(_field.verbose_name).capitalize(),
+                        required=False
+                    )
+                    if hasattr(self, 'instance') and self.instance.pk is not None:
+                        value = '{}'.format(getattr(self.instance, readonly_field)) \
+                            if getattr(self.instance, readonly_field) is not None else ""
+                    else:
+                        value = ''
+
+                self.fields[readonly_field].initial = value
+                self.initial[readonly_field] = value
 
     def webix_extra_clean(self, cleaned_data):
         for key, value in cleaned_data.items():
@@ -186,7 +198,7 @@ class BaseWebixMixin(object):
 
         rules = defaultdict(list)
         for name, field in self.fields.items():
-            if field.required and (not type(field) == forms.FileField or field.initial is not None):
+            if field.required and (not isinstance(field, forms.FileField) or field.initial is not None):
                 rules[self.add_prefix(name)].append({'rule': 'isNotEmpty'})
             if isinstance(field, forms.EmailField):
                 rules[self.add_prefix(name)].append({'rule': 'isEmail'})
@@ -196,7 +208,8 @@ class BaseWebixMixin(object):
                 rules[self.add_prefix(name)].append({'rule': 'isNumber', 'min': -six.MAXSIZE, 'max': six.MAXSIZE})
             elif isinstance(field, forms.IntegerField):
                 rules[self.add_prefix(name)].append({'rule': 'isInteger', 'min': -six.MAXSIZE, 'max': six.MAXSIZE})
-            elif isinstance(field, forms.CharField):
+            elif isinstance(field, forms.CharField) and \
+                (SimpleArrayField is not None and not isinstance(field, SimpleArrayField)):
                 rules[self.add_prefix(name)].append({'rule': 'isString', 'min': 0, 'max': field.max_length})
         return dict(rules)
 
@@ -208,8 +221,6 @@ class BaseWebixMixin(object):
         for name, field in self.fields.items():
             _pass = False
             label = force_text(field.label).capitalize()
-            if field.required:
-                label = '<strong>{}</strong>'.format(label)
             el = {
                 'view': 'text',
                 'label': label,
@@ -218,20 +229,23 @@ class BaseWebixMixin(object):
                 'labelWidth': 200,
                 'django_type_field': str(type(field).__name__),
             }
+            if field.required:
+                el['label'] = label = '<strong>{}</strong>'.format(label)
+                # el['required'] = True  # FIXME: problems with inlines
 
             if name in self.get_readonly_fields():
                 el.update({
                     'disabled': True,
-                    # 'id': '{}.{}'.format(self.webix_id, self[name].auto_id) # probabily old: disabled for has_change_permission=False
                 })
 
             # Get initial value
             if hasattr(self, 'cleaned_data'):
+
                 if type(field) == forms.models.ModelMultipleChoiceField:
                     initial = self.data.getlist(self.add_prefix(name), field.initial)
                 elif connection.vendor == 'postgresql' and isinstance(field, JSONField):
                     initial = self.data.get(self.add_prefix(name), None)
-                    if initial is not None:
+                    if initial is not None and initial != '':
                         initial = loads(initial)
                     else:
                         initial = field.initial
@@ -250,12 +264,11 @@ class BaseWebixMixin(object):
             # FloatField
             elif isinstance(field, forms.FloatField):
                 if initial is not None:
-                    el.update({'value': '{}'.format(initial).replace('.', ',')})
+                    el.update({'value': '{}'.format(formats.localize(initial, use_l10n=True))})
             # DecimalField
             elif isinstance(field, forms.DecimalField):
-                # el.update({'view':''})
                 if initial is not None:
-                    el.update({'value': '{}'.format(initial).replace('.', ',')})
+                    el.update({'value': '{}'.format(formats.localize(initial, use_l10n=True))})
             # IntegerField
             elif isinstance(field, forms.IntegerField):
                 el.update({
@@ -273,13 +286,30 @@ class BaseWebixMixin(object):
                 })
                 if initial is not None:
                     if isinstance(initial, six.string_types):
-                        el.update({
-                            'value': '{}'.format(initial).replace('-', ',').replace(' ', ',').replace(':', ',')
-                        })
+                        if settings.WEBIX_VERSION >= '7.0.0' and settings.WEBIX_VERSION < '8.0.0':
+                            el.update({
+                                'value': '2020-01-01 {}'.format(initial).replace('-', ',').replace(' ', ',').replace(
+                                    ':', ',')
+                            })
+                        else:
+                            el.update({
+                                'value': '{}'.format(initial).replace('-', ',').replace(' ', ',').replace(':', ',')
+                            })
                     elif callable(initial):
-                        el.update({'value': '{}'.format(initial().strftime('%H,%M'))})
+                        _value = initial()
+                        if not is_naive(_value):
+                            _value = make_naive(_value)
+                        if settings.WEBIX_VERSION >= '7.0.0' and settings.WEBIX_VERSION < '8.0.0':
+                            el.update({'value': '2020-01-01 {}'.format(_value.strftime('%H,%M'))})
+                        else:
+                            el.update({'value': '{}'.format(_value.strftime('%H,%M'))})
                     else:
-                        el.update({'value': '{}'.format(initial.strftime('%H,%M'))})
+                        if not is_naive(initial):
+                            initial = make_naive(initial)
+                        if settings.WEBIX_VERSION >= '7.0.0' and settings.WEBIX_VERSION < '8.0.0':
+                            el.update({'value': '2020-01-01 {}'.format(initial.strftime('%H,%M'))})
+                        else:
+                            el.update({'value': '{}'.format(initial.strftime('%H,%M'))})
             # DateField
             elif isinstance(field, forms.DateField):
                 el.update({
@@ -310,8 +340,13 @@ class BaseWebixMixin(object):
                             'value': '{}'.format(initial).replace('-', ',').replace(' ', ',').replace(':', ',')
                         })
                     elif callable(initial):
-                        el.update({'value': '{}'.format(initial().strftime('%Y,%m,%d,%H,%M'))})
+                        _value = initial()
+                        if not is_naive(_value):
+                            _value = make_naive(_value)
+                        el.update({'value': '{}'.format(_value.strftime('%Y,%m,%d,%H,%M'))})
                     else:
+                        if not is_naive(initial):
+                            initial = make_naive(initial)
                         el.update({'value': '{}'.format(initial.strftime('%Y,%m,%d,%H,%M'))})
             # BooleanField NullBooleanField
             elif isinstance(field, forms.NullBooleanField) or isinstance(field, forms.BooleanField):
@@ -344,19 +379,26 @@ class BaseWebixMixin(object):
                     'autosend': False,
                     'multiple': False,
                     'width': 100,
-                    'label': _('Upload new image')
+                    'label': _('Upload new image'),
+                    # 'on': {
+                    #     'onAfterFileAdd': "image_add('" + self[name].auto_id + "');"
+                    # }
                 })
+                delete_hidden = True
                 if initial:
+
                     img_small = get_thumbnail(initial, '150x100', quality=85)
                     img_big = get_thumbnail(initial, '500x400', quality=85)
                     key = randint(1, 100000)
                     _template_file = '<img src="{}"  onclick="image_modal(\'{}\',{},{},\'{}\')">'.format(
                         img_small.url, img_big.url, 500, 400, str(key)
                     )
+                    delete_hidden = False
                 else:
                     _template_file = ''
                 elements.update({
                     '{}_block'.format(self[name].html_name): {
+                        'id': 'block_' + self[name].auto_id,
                         'cols': [
                             {
                                 'name_label': name,
@@ -369,13 +411,28 @@ class BaseWebixMixin(object):
                             },
                             {
                                 'name_label': name,
-                                'id_label': 'preview_' + name,
+                                'id_label': 'preview_' + self[name].auto_id,
                                 'borderless': True,
                                 'template': _template_file,
                                 'height': 100,
                                 'width': 170
                             },
                             el,
+                            {
+                                'id': self[name].auto_id + '_clean',
+                                'name': self.add_prefix(name) + '_clean',
+                                'view': "toggle",
+                                'type': "icon",
+                                'offIcon': 'fas fa-trash-alt',
+                                'onIcon': 'fas fa-trash-alt',
+                                'offLabel': '',
+                                'onLabel': _('Eliminato'),
+                                'width': 100,
+                                'css': "webix_danger",
+                                'hidden': delete_hidden,
+                                'click': "delete_image('block_{}', '{}_clean')".format(self[name].auto_id,
+                                                                                       self[name].auto_id),
+                            },
                             {
                                 'borderless': True,
                                 'template': '',
@@ -455,10 +512,14 @@ class BaseWebixMixin(object):
             elif type(field) == forms.models.ModelMultipleChoiceField:
                 if settings.WEBIX_LICENSE != 'PRO':
                     raise ImproperlyConfigured(
-                        _("MultipleChoiceField is available only with PRO webix license"))
+                        _("MultipleChoiceField is available only with PRO webix license")
+                    )
                 if initial is not None:
                     el.update({
-                        'value': ','.join([str(i.pk) if isinstance(i, models.Model) else str(i) for i in initial])
+                        'value': ','.join([
+                            str(getattr(i, field.to_field_name or 'pk')) if isinstance(i, models.Model) else str(i)
+                            for i in initial
+                        ])
                     })
                 count = field.queryset.count()
 
@@ -496,18 +557,11 @@ class BaseWebixMixin(object):
                     if 'value' in el:
                         _vals = el['value'].split(",")
                     for _val in [i for i in _vals if i != ''.strip()]:
-                        if field.to_field_name:
-                            record = field.queryset.get(**{field.to_field_name: _val})
-                            el['options']['body']['data'].append({
-                                'id': '{}'.format(getattr(record, field.to_field_name)),
-                                'value': '{}'.format(record)
-                            })
-                        else:
-                            record = field.queryset.get(pk=_val)
-                            el['options']['body']['data'].append({
-                                'id': '{}'.format(record.pk),
-                                'value': '{}'.format(record)
-                            })
+                        record = field.queryset.get(**{field.to_field_name or 'pk': _val})
+                        el['options']['body']['data'].append({
+                            'id': '{}'.format(getattr(record, field.to_field_name or 'pk')),
+                            'value': '{}'.format(record)
+                        })
 
                 # regular field without autocomplete
                 else:
@@ -516,10 +570,11 @@ class BaseWebixMixin(object):
                         'selectAll': True,
                         'placeholder': _('Click to select'),
                         'options': {
+                            'selectAll': True,
                             'dynamic': True,
                             'body': {
                                 'data': self._add_null_choice([{
-                                    'id': '{}'.format(i.pk),
+                                    'id': '{}'.format(getattr(i, field.to_field_name or 'pk')),
                                     'value': '{}'.format(i)
                                 } for i in field.queryset])
                             }
@@ -527,11 +582,15 @@ class BaseWebixMixin(object):
                     })
                     # Default if is required and there are only one option
                     if field.required and initial is None and len(field.queryset) == 1:
-                        el.update({'value': '{}'.format(field.queryset.first().pk)})
+                        el.update({'value': '{}'.format(getattr(field.queryset.first(), field.to_field_name or 'pk'))})
             # ModelChoiceField
             elif isinstance(field, forms.models.ModelChoiceField):
                 if initial is not None:
-                    el.update({'value': initial.pk if isinstance(initial, models.Model) else str(initial)})
+                    el.update({
+                        'value': str(getattr(initial, field.to_field_name or 'pk'))
+                        if isinstance(initial, models.Model) else
+                        str(initial)
+                    })
                 count = field.queryset.count()
 
                 if count > self.min_count_suggest and \
@@ -564,23 +623,16 @@ class BaseWebixMixin(object):
                         }
                     })
                     if 'value' in el and el['value'] != '':  # and int(el['value']) > 0:
-                        if field.to_field_name:
-                            record = field.queryset.get(**{field.to_field_name: el['value']})
-                            el['suggest']['body']['data'] = [{
-                                'id': '{}'.format(getattr(record, field.to_field_name)),
-                                'value': '{}'.format(record)
-                            }]
-                        else:
-                            record = field.queryset.get(pk=el['value'])
-                            el['suggest']['body']['data'] = [{
-                                'id': '{}'.format(record.pk),
-                                'value': '{}'.format(record)
-                            }]
+                        record = field.queryset.get(**{field.to_field_name or 'pk': el['value']})
+                        el['suggest']['body']['data'] = [{
+                            'id': '{}'.format(getattr(record, field.to_field_name or 'pk')),
+                            'value': '{}'.format(record)
+                        }]
 
                 # regular field without autocomplete
                 elif count <= 6:
                     choices = self._add_null_choice([{
-                        'id': '{}'.format(i.pk),
+                        'id': '{}'.format(getattr(i, field.to_field_name or 'pk')),
                         'value': '{}'.format(i)
                     } for i in field.queryset])
                     if not field.required:
@@ -593,10 +645,10 @@ class BaseWebixMixin(object):
                     })
                     # Default if is required and there are only one option
                     if field.required and initial is None and len(field.queryset) == 1:
-                        el.update({'value': '{}'.format(field.queryset.first().pk)})
+                        el.update({'value': '{}'.format(getattr(field.queryset.first(), field.to_field_name or 'pk'))})
                 else:
                     choices = self._add_null_choice([{
-                        'id': '{}'.format(i.pk),
+                        'id': '{}'.format(getattr(i, field.to_field_name or 'pk')),
                         'value': '{}'.format(i)
                     } for i in field.queryset])
                     if not field.required:
@@ -625,7 +677,6 @@ class BaseWebixMixin(object):
                 # Default if is required and there are only one option
                 if field.required and initial is None and len(field.choices) == 1:
                     el.update({'value': '{}'.format(field.choices[0][0])})
-
             # JSONField (postgresql)
             elif connection.vendor == 'postgresql' and isinstance(field, JSONField):
                 if isinstance(field.widget, forms.widgets.Textarea):
@@ -634,7 +685,6 @@ class BaseWebixMixin(object):
                     })
                 if initial is not None:
                     el.update({'value': dumps(initial)})
-
             # SimpleArrayField (postgresql)
             elif connection.vendor == 'postgresql' and \
                 SimpleArrayField is not None and \
@@ -652,9 +702,29 @@ class BaseWebixMixin(object):
                                 'value': ', '.join([i.strftime('%Y-%m-%d') for i in initial]),
                                 'orig_value': ', '.join([i.strftime('%Y-%m-%d') for i in initial]),
                             })
+                elif hasattr(field, 'base_field') and isinstance(field.base_field, forms.fields.TypedChoiceField) and \
+                    hasattr(field.base_field, 'choices'):
+                    el.update({
+                        "view": "multicombo",
+                        'selectAll': True,
+                        'placeholder': _('Click to select'),
+                        'options': {
+                            'selectAll': True,
+                            'dynamic': True,
+                            'body': {
+                                'data': self._add_null_choice([{
+                                    'id': '{}'.format(k),
+                                    'value': '{}'.format(v)
+                                } for k, v in field.base_field.choices])
+                            }
+                        },
+                    })
+                    if initial is not None:
+                        el.update({
+                            'value': ','.join([str(i.pk) if isinstance(i, models.Model) else str(i) for i in initial])
+                        })
                 elif initial is not None:
                     el.update({'value': initial})
-
             # CharField
             elif isinstance(field, forms.CharField):
                 if isinstance(field.widget, forms.widgets.Textarea):
@@ -663,7 +733,6 @@ class BaseWebixMixin(object):
                     })
                 if initial is not None:
                     el.update({'value': initial})
-
             # GeoFields
             elif GeometryField is not None and isinstance(field, GeometryField):
                 el.update({
@@ -677,15 +746,14 @@ class BaseWebixMixin(object):
                         el.update({'value': str(initial)})
                     else:
                         raise Exception(_('Initial value {} for geo field {} is not supported').format(initial, field))
-
             # InlineForeignKey
             elif isinstance(field, forms.models.InlineForeignKeyField):
                 pass
-
+            # Field not supported
             else:
                 raise Exception(_('Type of field {} not supported').format(field))
 
-            # widget RadioSelect
+            # Widget RadioSelect
             if isinstance(field.widget, forms.RadioSelect):
                 _choices = field.choices if hasattr(field, 'choices') else field.widget.choices
                 el.update({
@@ -701,8 +769,12 @@ class BaseWebixMixin(object):
                 if field.required and initial is None and len(_choices) == 1:
                     el.update({'value': '{}'.format(_choices[0][0])})
 
-            # widget Hidden Fields
-            if type(field.widget) == forms.widgets.HiddenInput:
+            # Widget RadioSelect
+            if isinstance(field.widget, forms.PasswordInput):
+                el.update({"type": "password"})
+
+            # Widget Hidden Fields
+            if isinstance(field.widget, forms.HiddenInput):
                 el.update({'hidden': True})
 
             # Delete checkbox hidden
@@ -790,8 +862,6 @@ class BaseWebixMixin(object):
         return fs.values()
 
 
-####################### BaseWebixForm AND BaseWebixModelForm #######################
-
 class BaseWebixForm(forms.BaseForm, BaseWebixMixin):
 
     def __init__(self, data=None, files=None, auto_id='id_%s', prefix=None,
@@ -881,9 +951,6 @@ class BaseWebixModelForm(forms.BaseModelForm, BaseWebixMixin):
         return capfirst(self.Meta.model._meta.verbose_name)
 
 
-####################### WebixForm AND WebixModelForm #######################
-
-
 class WebixForm(six.with_metaclass(DeclarativeFieldsMetaclass, BaseWebixForm)):
     pass
 
