@@ -1,4 +1,5 @@
-{% load django_webix_utils static i18n %}
+{% load django_webix_utils static i18n filtersmerger_utils %}
+{% get_request_filter_params %}
 
 {% block webix_content %}
 
@@ -6,90 +7,107 @@
 webix.ui([], $$("{{ webix_container_id }}"));
 {% endblock %}
 
+{% if model %}
+initWebixFilterPrefix('{{ model|getattr:'_meta'|getattr:'app_label'}}.{{ model|getattr:'_meta'|getattr:'model_name'}}');
+{% endif %}
+
 {% block filter_options %}
     {% for field_name, choices_filter in choices_filters.items %}
         var {{ field_name }}_options = {{ choices_filter|safe }}
     {% endfor %}
 {% endblock %}
 
-{% if geo_filter %}
-var {{ view_prefix }}geo_filter = '{{ geo_filter|escapejs }}';
-{% else %}
-var {{ view_prefix }}geo_filter = undefined;
-{% endif %}
-
-{% if sql_filters %}
-var {{ view_prefix }}sql_filters = '{{ sql_filters|escapejs }}';
-{% else %}
-var {{ view_prefix }}sql_filters = undefined;
-{% endif %}
-
-{% if qsets_locked_filters %}
-var {{ view_prefix }}locked_filters = '{{ qsets_locked_filters|escapejs }}';
-{% else %}
-var {{ view_prefix }}locked_filters = undefined;
-{% endif %}
-
-var {{ view_prefix }}django_webix_filters = [{% for f in django_webix_filters %}{{ f.pk }}{% if not forloop.last %},{% endif %}{% endfor %}];
-
-        {% block toolbar_navigation %}
-        {% if title %}
-        $$("{{ webix_container_id }}").addView({
-            id: '{{ view_prefix }}main_toolbar_navigation',
-            view: "toolbar",
-            margin: 5,
-            cols: [
-                {
-                    id: '{{ view_prefix }}filter',
-                    view: "text",
-                    value: "1",
-                    hidden: true,
-                },
-                {},
-                {
-                    view: "template",
-                    type: "header",
-                    borderless: true,
-                    template: '<div style="width:100%; text-align:center;"><strong>{{ title }}</strong></div>'
-                },
-                {},
-                {
-                    view: "button",
-                    label: "{{_("Extra filters removal")|escapejs}}",
-                    type: "icon",
-                    width: 200,
-                    hidden: ({{ view_prefix }}locked_filters==undefined) && ({{ view_prefix }}sql_filters==undefined),
-                    icon: "far fa-trash-alt",
-                    click: function (id, event) {
-                        load_js('{{ url_list }}', undefined, undefined, undefined, undefined, undefined, undefined, abortAllPending=true);
-                    }
-                },
-                {
-                    view: "button",
-                    label: "{{_("Geographic filter removal")|escapejs}}",
-                    type: "icon",
-                    width: 200,
-                    hidden: ({{ view_prefix }}geo_filter==undefined),
-                    icon: "far fa-trash-alt",
-                    click: function (id, event) {
-                        load_js('{{ url_list }}', undefined, undefined, undefined, undefined, undefined, undefined, abortAllPending=true);
-                    }
-                },
-                {% if is_installed_django_webix_filter %}
-                {
-                    view: "button",
-                    label: "{{_("Advanced filter")|escapejs}} <div class='webix_badge' style='background-color: #ff8839 !important;' id='{{ view_prefix }}django_webix_filter_counter'>0</div>",
-                    type: "icon",
-                    width: 180,
-                    //hidden: django_webix_filters.length==0,
-                    icon: "far fa-filter",
-                    click: function (id, event) { // (lnk, hide, area, method, data)
-                        load_js('{% url 'django_webix_filter.webixfilter.list_model' app_label=app_label model_name=module_name %}?_popup', undefined, undefined, undefined, undefined, undefined, undefined, abortAllPending=true);
-                    }
-                },
+{% block toolbar_navigation %}
+{% if title %}
+$$("{{ webix_container_id }}").addView({
+    id: '{{ view_prefix }}main_toolbar_navigation',
+    view: "toolbar",
+    margin: 5,
+    cols: [
+        {
+            id: '{{ view_prefix }}filter',
+            view: "text",
+            value: "0", // disable by default
+            hidden: true,
+        },
+        {},
+        {
+            view: "template",
+            type: "header",
+            borderless: true,
+            template: '<div style="width:100%; text-align:center;"><strong>{{ title }}</strong></div>'
+        },
+        {},
+        {
+            id: '{{ view_prefix }}_filter_locked_sql',
+            view: "button",
+            label: "{{_("Extra filters removal")|escapejs}}",
+            type: "icon",
+            width: 200,
+            hidden: true,
+            icon: "far fa-trash-alt",
+            click: function (id, event) {
+                {% if model %}
+                {% for key, param in extra_filter_params.items %}{% if param %}
+                setWebixFilter('{{ model|getattr:'_meta'|getattr:'app_label'}}.{{ model|getattr:'_meta'|getattr:'model_name'}}', '{{ param }}', null);
+                {% endif %}{% endfor %}
                 {% endif %}
-            ]
-        });
+                load_js('{{ url_list }}', undefined, undefined, undefined, undefined, undefined, undefined, abortAllPending = true);
+            }
+        },
+        {
+            id: '{{ view_prefix }}_filter_geo',
+            view: "button",
+            label: "{{_("Geographic filter removal")|escapejs}}",
+            type: "icon",
+            width: 200,
+            hidden: true,
+            icon: "far fa-trash-alt",
+            click: function (id, event) {
+                {% if model %}
+                {% for key, param in spatial_filter_params.items %}{% if param %}
+                setWebixFilter('{{ model|getattr:'_meta'|getattr:'app_label'}}.{{ model|getattr:'_meta'|getattr:'model_name'}}', '{{ param }}', null);
+                {% endif %}{% endfor %}
+                {% endif %}
+                load_js('{{ url_list }}', undefined, undefined, undefined, undefined, undefined, undefined, abortAllPending = true);
+            }
+        },
+        {% if is_installed_django_webix_filter %}
+        {
+            id: '{{ view_prefix }}_filter_advanced',
+            view: "button",
+            label: "{{_("Advanced filter")|escapejs}} <div class='webix_badge' style='background-color: #ff8839 !important;' id='{{ view_prefix }}django_webix_filter_counter'>0</div>",
+            type: "icon",
+            width: 180,
+            icon: "far fa-filter",
+            click: function (id, event) { // (lnk, hide, area, method, data)
+                load_js('{% url 'django_webix_filter.webixfilter.list_model' app_label=app_label model_name=module_name %}?_popup', undefined, undefined, undefined, undefined, undefined, undefined, abortAllPending = true);
+            }
+        },
+        {% endif %}
+
+    ]
+});
+
+{% if model %}
+if (
+    false
+    {% for key, param in extra_filter_params.items %}{% if param %}
+    || (webixAppliedFilters['{{ model|getattr:'_meta'|getattr:'app_label'}}.{{ model|getattr:'_meta'|getattr:'model_name'}}']['{{ param }}'] != null)
+    {% endif %}{% endfor %}
+) {
+    $$('{{ view_prefix }}_filter_locked_sql').show();
+}
+if (
+    false
+    {% for key, param in spatial_filter_params.items %}{% if param %}
+    || (webixAppliedFilters['{{ model|getattr:'_meta'|getattr:'app_label'}}.{{ model|getattr:'_meta'|getattr:'model_name'}}']['{{ param }}'] != null)
+    {% endif %}{% endfor %}
+) {
+    $$('{{ view_prefix }}_filter_geo').show();
+}
+{% endif %}
+
 {% endif %}
 {% endblock %}
 
@@ -139,7 +157,11 @@ function {{ view_prefix }}get_filters_qsets() {
                         }
                     })
                 } else {
-                    qsets.push({'path': el.id + '__' + el.serverFilterType, 'val': value});
+                    if ((el.serverFilterType=='') || (el.serverFilterType==undefined)){
+                        qsets.push({'path': el.id, 'val': value});
+                    } else{
+                        qsets.push({'path': el.id + '__' + el.serverFilterType, 'val': value});
+                    }
                 }
             }
         }
@@ -153,11 +175,113 @@ function {{ view_prefix }}get_filters_qsets() {
 
 {% block datatable %}
 
-function {{ view_prefix }}apply_filters(){
-  $('#{{ view_prefix }}django_webix_filter_counter').text({{ view_prefix }}django_webix_filters.length);
-  $$('{{ view_prefix }}filter').setValue('1');
-  $$('{{ view_prefix }}datatable').filterByAll();
-  }
+function {{ view_prefix }}is_active_otf_filter(){
+    var key = '{{ model_name }}';
+    if(typeof otf_filter !== "undefined") {
+        if (otf_filter[key] != undefined) {
+            if (otf_filter[key]['list'] != undefined) {
+                if (otf_filter[key]['list']['active']) {
+                    return true;
+                }
+            }
+        }
+    }
+    return false;
+}
+
+function {{ view_prefix }}is_present_otf_filter(){
+    var key = '{{ model_name }}';
+    if(typeof otf_filter !== "undefined") {
+        if (otf_filter[key] != undefined) {
+            if (otf_filter[key]['list'] != undefined) {
+                if (otf_filter[key]['list']['json'] != undefined) {
+                    return true;
+                }
+            }
+        }
+    }
+    return false;
+}
+
+function {{ view_prefix }}get_otf_filter(){
+    var key = '{{ model_name }}';
+    if(typeof otf_filter !== "undefined") {
+        if (otf_filter[key] != undefined) {
+            if (otf_filter[key]['list'] != undefined) {
+                if (otf_filter[key]['list']['json'] != undefined) {
+                    return otf_filter[key]['list']['json'];
+                }
+            }
+        }
+    }
+    return undefined;
+}
+
+function {{ view_prefix }}deactivate_otf_filter(){
+    var key = '{{ model_name }}';
+    if(typeof otf_filter !== "undefined") {
+        if (otf_filter[key] != undefined) {
+            if (otf_filter[key]['list'] != undefined) {
+                otf_filter[key]['list']['active'] = false;
+                return true;
+            }
+        }
+    }
+    return false;
+}
+
+function {{ view_prefix }}apply_filters() {
+    var extra = ''
+    if({{ view_prefix }}is_active_otf_filter()){
+        extra = '+';
+    }
+    var advanced_filter_count = '0';
+    {% with param='DjangoAdvancedWebixFilter'|request_filter_param %}
+    {% if param %}
+    var advanced_filter = webixAppliedFilters['{{ model_name }}']['{{ param }}'];
+    if (advanced_filter){
+        advanced_filter_count = advanced_filter.split(',').length
+    }
+    {% endif %}
+    {% endwith %}
+    if(extra != '' && advanced_filter_count == '0'){
+        advanced_filter_count = '';
+    }
+    $('#{{ view_prefix }}django_webix_filter_counter').text(advanced_filter_count + extra);
+    $$('{{ view_prefix }}filter').setValue('1');
+    $$('{{ view_prefix }}datatable').filterByAll();
+}
+
+{% if is_enable_column_webgis %}
+{% for layer in layers %}
+function custom_button_geo_{{layer.codename}}(obj, common, value) {
+    if (obj.{{ layer.geofieldname }}_available==true)
+        return '<div title="{{_("Go to map")|escapejs}} ({{layer.layername}})"><i style="cursor:pointer" class="webix_icon far fa-map-marker-alt"></i></div>';
+    else
+        return ''
+}
+{% endfor %}
+{% endif %}
+
+{%  if is_json_loading %}
+$$("{{ webix_container_id }}").addView({
+    template: "{common.first()} {common.prev()} {common.pages()} {common.next()} {common.last()}",
+    id: "datatable_paging_{{ model_name }}", // the container to place the pager controls into
+    view: "pager",
+    group: 5, // buttons for next amd back
+    // must to be the same of url request because managed from interface
+    size: {{ paginate_count_default }},
+    page: 0,
+    on: {
+        onBeforePageChange: function (new_page, old_page) {
+            if ($('input[name="{{ view_prefix }}master_checkbox"]').prop("checked") == true) {
+                $('input[name="{{ view_prefix }}master_checkbox"]').click();
+                {{ view_prefix }}update_counter();
+            }
+        }
+    }
+})
+{% endif %}
 
 $$("{{ webix_container_id }}").addView({
     id: '{{ view_prefix }}datatable',
@@ -192,18 +316,34 @@ $$("{{ webix_container_id }}").addView({
         {% endfor %}
         {% endblock %}
         {% block datatable_columns_commands %}
+        {% if is_enable_column_webgis %}
+            {% for layer in layers %}
+                {
+                    id: "cmd_gotomap_{{layer.codename}}",
+                    header: "",
+                    headermenu: false,
+                    width:40,
+                    tooltip: false,
+                    template: custom_button_geo_{{layer.codename}}
+                },
+            {% endfor %}
+        {% endif %}
         {
             id: "cmd_cp",
+            {% if is_json_loading %}
             header: [
                 {
                     text: "",
                     colspan: 2
                 },
                 {
-                    text: '<div class="webix_view webix_control webix_el_button webix_secondary"><div title="{{_("Apply filters") }}" class="webix_el_box"><button type="button" class="webix_button webix_img_btn" style="line-height:24px;" onclick="{{ view_prefix }}apply_filters(\'{{ model_name }}\');"> {{_("Filter") }} </button></div></div>',
+                    text: '<div class="webix_view webix_control webix_el_button webix_secondary"><div title="{{_("Apply filters") }}" class="webix_el_box"><button id="button_filter_datatable" type="button" class="webix_button webix_img_btn" style="line-height:24px;" onclick="{{ view_prefix }}apply_filters(\'{{ model_name }}\');"> {{_("Filter") }} </button></div></div>',
                     colspan: 2
                 }
             ],
+            {% else %}
+            header: "",
+            {% endif %}
             headermenu: false,
             width:40,
             tooltip: false,
@@ -226,7 +366,6 @@ $$("{{ webix_container_id }}").addView({
         $proxy: true,
         source: "{{ url_list }}{% if '?' in url_list %}&{% else %}?{% endif %}json",
         load: function (view, params) {
-
             // elaborate paging
             var _count = {{ paginate_count_default }};
             var _start = 0;
@@ -236,30 +375,16 @@ $$("{{ webix_container_id }}").addView({
             if ((params) && (params.start)) {
                 _start = params.start;
             }
-            var _params = {
-                'start': _start,
-                'count': _count,
-            }
+            {% if model %}
+            var _params = webixAppliedFilters['{{ model|getattr:'_meta'|getattr:'app_label'}}.{{ model|getattr:'_meta'|getattr:'model_name'}}'];
+            {% else %}
+            var _params = {};
+            {% endif %}
+            _params.start = _start;
+            _params.count = _count;
             // elaborate filters
             _params.filters = JSON.stringify( {{ view_prefix }}get_filters_qsets() );
 
-            if ({{ view_prefix }}geo_filter!=undefined) {
-                _params.geo_filter = {{ view_prefix }}geo_filter;
-            }
-
-            if ({{ view_prefix }}sql_filters!=undefined) {
-                _params.sql_filters = {{ view_prefix }}sql_filters;
-            }
-
-            if ({{ view_prefix }}locked_filters!=undefined) {
-                _params.locked_filters = {{ view_prefix }}locked_filters;
-            }
-
-            if ({{ view_prefix }}django_webix_filters!=undefined) {
-                _params.django_webix_filters = {{ view_prefix }}django_webix_filters;
-            }
-
-
             // elaborate sort
             sort = $$('{{ view_prefix }}datatable').getState().sort;
             if (sort != undefined) {
@@ -301,14 +426,16 @@ $$("{{ webix_container_id }}").addView({
         onCheck: function (row, column, state) {
             {{ view_prefix }}update_counter();
         },
+    {% if is_json_loading %}
         onBeforeFilter: function (id) {
             if ($$('{{ view_prefix }}filter').getValue() == '0') {
                 return false
             } else {
-                $$("{{ view_prefix }}datatable").getFilter(id).disabled = true;
+                if ($$("{{ view_prefix }}datatable").getFilter(id)) {
+                    $$("{{ view_prefix }}datatable").getFilter(id).disabled = true;
+                }
             }
         },
-
         onAfterFilter: function () {
             var columns = this.config.columns;
             columns.forEach(function (obj) {
@@ -318,6 +445,7 @@ $$("{{ webix_container_id }}").addView({
             });
             $$('{{ view_prefix }}filter').setValue('0');
         },
+    {% endif %}
         onBeforeLoad: function () {
             this.showOverlay("<img src='{% static 'django_webix/loading.gif' %}'>");
         },
@@ -325,6 +453,8 @@ $$("{{ webix_container_id }}").addView({
             {% if not is_json_loading %}
             $$('{{ view_prefix }}datatable').view_count = {{ view_prefix }}objects_list.length;
             $$('{{ view_prefix }}datatable').view_count_total = {{ view_prefix }}objects_list.length;
+            {% else %}
+            $$('{{ view_prefix }}filter').setValue('0');
             {% endif %}
             this.hideOverlay();
         },
@@ -344,8 +474,15 @@ $$("{{ webix_container_id }}").addView({
             if ((id.column == '{{ field.field_name }}') || (id.column == '{{ field.column_name }}')) {
                 {{ field.click_action|safe }};
             } else
-                    {% endif %}
-                    {% endfor %}
+            {% endif %}
+            {% endfor %}
+            {% if is_enable_column_webgis %}
+            {% for layer in layers %}
+            if ((id.column == 'cmd_gotomap_{{layer.codename}}')) {
+                $$("map").goToWebgisPk('{{layer.qxsname}}', '{{ pk_field_name }}', el.id);
+            } else
+            {% endfor %}
+            {% endif %}
             if (id.column == 'cmd_cp') {
                 {% block cmd_cp_click %}
                     {% if is_enable_column_copy %}
@@ -370,7 +507,7 @@ $$("{{ webix_container_id }}").addView({
             {% endblock %}
         }
     }
-});
+},1);
 
 {% block footer %}
     {% if is_enable_footer %}
@@ -386,155 +523,22 @@ $$("{{ webix_container_id }}").addView({
 {% endblock %}
 
 // disable filter on first request
-$$('{{ view_prefix }}filter').setValue('0');
+// $$('{{ view_prefix }}filter').setValue('0');
 
-{%  if is_json_loading %}
-$$("{{ webix_container_id }}").addView({
-    template: "{common.first()} {common.prev()} {common.pages()} {common.next()} {common.last()}",
-    id: "datatable_paging_{{ model_name }}", // the container to place the pager controls into
-    view: "pager",
-    group: 5, // buttons for next amd back
-    // must to be the same of url request because managed from interface
-    size: {{ paginate_count_default }},
-    page: 0,
-    on: {
-        onBeforePageChange: function (new_page, old_page) {
-            if ($('input[name="{{ view_prefix }}master_checkbox"]').prop("checked") == true) {
-                $('input[name="{{ view_prefix }}master_checkbox"]').click();
-                {{ view_prefix }}update_counter();
-            }
-        }
-    }
-})
-{% endif %}
 
 {% endblock %}
 
 {% block toolbar_list %}
 
-function _{{ view_prefix }}action_execute(action, ids, all, response_type, short_description, modal_title, modal_ok, modal_cancel) {
-    /*
-    action (required) = action_key to be executed
-    ids (required) = list of selected elements ids
-    all (required) = boolean if all elements are selected
-    response_type (required) = ['script', 'json', 'blank']
-    short_description (not required)
-    modal_title (required) = text to show in modal choices execution
-    modal_ok (not required)
-    modal_cancel (not required)
-    */
-    webix.confirm({
-        title: short_description,
-        ok: modal_ok,
-        cancel: modal_cancel,
-        text:  modal_title + "</br><b>" + ids.length + " {{_("elements")|escapejs}}</b> {{_("selected")|escapejs}}",
-        callback: function (confirm) {
-            if (confirm==true) {
-                $$('{{ view_prefix }}datatable').showOverlay("<img src='{% static 'django_webix/loading.gif' %}'>");
-                if ((response_type=='json') || (response_type=='script')){
-                    _params = {
-                        'action': action,
-                        'filters': JSON.stringify({{ view_prefix }}get_filters_qsets()),
-                        'csrfmiddlewaretoken': getCookie('csrftoken')
-                    };
-                    if (all == false) {
-                        _params['ids'] = ids.join(',');
-                    }
-                    $.ajax({
-                        url: "{{ url_list }}",
-                        type: "POST",
-                        dataType: response_type,
-                        data: _params,
-                        error: function (data) {
-                            webix.message({
-                                text: "{{_("Action is not executable")|escapejs}}",
-                                type: "error",
-                                expire: 10000
-                            });
-                        },
-                        success: function (data) { // TODO gestire response
-                            if (data.status==true) {
-                                webix.message({
-                                    text: data.message,
-                                    type: "info",
-                                    expire: 5000
-                                });
-                                if (data.redirect_url!=null){
-                                    load_js(data.redirect_url, undefined, undefined, undefined, undefined, undefined, undefined, abortAllPending=true);
-                                    }
-                            } else {
-                                webix.message({
-                                    text: "{{_("Something gone wrong")|escapejs}}",
-                                    type: "error",
-                                    expire: 10000
-                                });
-                            }
-                        },
-                    });
-                } else if (response_type=='blank'){
-                    var form = document.createElement("form");
-                    form.setAttribute("method", "post");
-                    form.setAttribute("action", "{{ url_list }}");
-                    form.setAttribute("target", "view");
-                    _fields = [
-                        ['action',action],
-                        ['filters', JSON.stringify( {{ view_prefix }}get_filters_qsets() )],
-                        ['csrfmiddlewaretoken',getCookie('csrftoken')]
-                    ];
-                    if (all==false){
-                        _fields.push(['ids',ids.join(',')])
-                    }
-                    $.each(_fields, function( index, value ) {
-                        var hiddenField = document.createElement("input");
-                            hiddenField.setAttribute("type", "hidden");
-                            hiddenField.setAttribute("name", value[0]);
-                            hiddenField.setAttribute("value", value[1]);
-                            form.appendChild(hiddenField);
-                        });
-                    document.body.appendChild(form);
-                    window.open('', 'view');
-                    form.submit();
-                } else {
-
-                }
-                $$('{{ view_prefix }}datatable').hideOverlay();
-            }
-        }
-    })
-}
+{% include "django_webix/include/actions_utils.js" %}
 
 {% block toolbar_list_actions %}
-{% if is_enable_actions %}
-
-var {{ view_prefix }}actions_list = [
-    {% for action_key,action in actions.items %}
-    {id: '{{ action_key }}', value: '{{action.short_description}}'}{% if not forloop.last %}, {% endif %}
-    {% endfor %}
-];
-
-function {{ view_prefix }}actions_execute(action, ids, all) {
-    {% for action_key, action in actions.items %} if (action=='{{ action_key }}') {
-        _{{ view_prefix }}action_execute(
-                '{{ action_key }}',
-                ids,
-                all,
-                '{{ action.response_type }}',
-                '{{ action.short_description }}',
-                '{{ action.modal_title }}',
-                '{{ action.modal_ok }}',
-                '{{ action.modal_cancel }}'
-        )
-    } {% if not forloop.last %} else {% endif %} {% endfor %}
-
-}
-{% else %}
-var {{ view_prefix }}actions_list = undefined;
-var {{ view_prefix }}actions_execute = undefined;
-{% endif %}
+{% include "django_webix/include/actions_list.js" %}
 {% endblock %}
 
 {% include "django_webix/include/toolbar_list.js" %}
 {% endblock %}
 
+{{ view_prefix }}apply_filters();
 {% block extrajs_post %}{% endblock %}
 {% endblock %}
