@@ -1,18 +1,18 @@
-from django_webix.views.generic.base import WebixTemplateView
-from django_webix.views.generic.detail import WebixDetailView
-from django_webix.views.generic.list import WebixListView, WebixTemplateListView
+# -*- coding: utf-8 -*-
+
+from django_webix.views.generic.base import WebixTemplateView, WebixFormView
 from django_webix.views.generic.create_update import (
-    WebixCreateView, WebixUpdateView, WebixCreateWithInlinesView, WebixCreateWithInlinesUnmergedView,
-    WebixUpdateWithInlinesView, WebixUpdateWithInlinesUnmergedView
+    WebixCreateView, WebixUpdateView
 )
 from django_webix.views.generic.delete import WebixDeleteView
+from django_webix.views.generic.detail import WebixDetailView
+from django_webix.views.generic.list import WebixListView, WebixTemplateListView
 
 __all__ = [
     # model views
     'WebixCreateView', 'WebixUpdateView', 'WebixDetailView', 'WebixListView', 'WebixDeleteView',
     # template views
     'WebixTemplateView', 'WebixTemplateListView',
-    # deprecated
-    'WebixCreateWithInlinesView', 'WebixCreateWithInlinesUnmergedView', 'WebixUpdateWithInlinesView',
-    'WebixUpdateWithInlinesUnmergedView'
+    # form views
+    'WebixFormView',
 ]
