@@ -1,5 +1,8 @@
+# -*- coding: utf-8 -*-
+
 from django.http import JsonResponse
-from django.utils.translation import ugettext as _
+from django.utils.translation import gettext as _
+
 from django_webix.views.generic.decorators import action_config
 
 
